/*********************************************************************
 * Software License Agreement (BSD License)
 *
 *  Copyright (c) 2011, Neobotix GmbH
 *  All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above
 *     copyright notice, this list of conditions and the following
 *     disclaimer in the documentation and/or other materials provided
 *     with the distribution.
 *   * Neither the name of the Neobotix nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 *  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 *  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 *  ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 *  POSSIBILITY OF SUCH DAMAGE.
 *********************************************************************/

// general includes
#include <math.h>

// Headers provided by other cob-packages
#include <cob_generic_can/CanESD.h>
#include <cob_generic_can/CanPeakSys.h>
#include <cob_generic_can/CanPeakSysUSB.h>
#include <neo_base_drive_mpo_500/NeoCtrlPltfMpo500.h>


//-----------------------------------------------

CanCtrlPltfCOb3::CanCtrlPltfCOb3(std::string iniDirectory)
{	
	sIniDirectory = iniDirectory;
	IniFile iniFile;
	iniFile.SetFileName(sIniDirectory + "Platform.ini", "PltfHardwareCoB3.h");

	// get max Joint-Velocities (in rad/s) for Steer- and Drive-Joint
	iniFile.GetKeyInt("Config", "NumberOfMotors", &m_iNumMotors, true);
	iniFile.GetKeyInt("Config", "NumberOfWheels", &m_iNumDrives, true);

/*	if(m_iNumMotors < 2 || m_iNumMotors > 8) {
		m_iNumMotors = 8;
		m_iNumDrives = 4;
	}
*/
	// ------------- first of all set used CanItf
	m_pCanCtrl = NULL;

	// ------------- init hardware-specific vectors and set default values
	m_vpMotor.resize(m_iNumMotors);

	for(int i=0; i<m_iNumMotors; i++)
	{
		m_vpMotor[i] = NULL;
	}

	m_viMotorID.resize(m_iNumMotors);

//	m_viMotorID.resize(8);
	if(m_iNumMotors >= 1)
		m_viMotorID[0] = CANNODE_WHEEL1DRIVEMOTOR;
	if(m_iNumMotors >= 2)
		m_viMotorID[1] = CANNODE_WHEEL2DRIVEMOTOR;
	if(m_iNumMotors >= 3)
		m_viMotorID[2] = CANNODE_WHEEL3DRIVEMOTOR;
	if(m_iNumMotors >= 4)
		m_viMotorID[3] = CANNODE_WHEEL4DRIVEMOTOR;


	// ------------- parameters
	m_Param.dCanTimeout = 7;

	if(m_iNumMotors >= 1)
		m_Param.iHasWheel1DriveMotor = 0;
	if(m_iNumMotors >= 2)
		m_Param.iHasWheel2DriveMotor = 0;
	if(m_iNumMotors >= 3)
		m_Param.iHasWheel3DriveMotor = 0;
	if(m_iNumMotors >= 4)
		m_Param.iHasWheel4DriveMotor = 0;

	m_bWatchdogErr = false;
	
	// ------------ CanIds
	
	// ------------ For CanOpen (Harmonica)
	// Wheel 1
	// can adresse motor 1
	if(m_iNumMotors >= 1)
	{
		m_CanOpenIDParam.TxPDO1_W1Drive = 0x182;
		m_CanOpenIDParam.TxPDO2_W1Drive = 0x282;
		m_CanOpenIDParam.RxPDO2_W1Drive = 0x302;
		m_CanOpenIDParam.TxSDO_W1Drive = 0x582;
		m_CanOpenIDParam.RxSDO_W1Drive = 0x602;
	}
	// can adresse motor 2
	if(m_iNumMotors >= 2)
	{
		m_CanOpenIDParam.TxPDO1_W2Drive = 0x183;
		m_CanOpenIDParam.TxPDO2_W2Drive = 0x283;
		m_CanOpenIDParam.RxPDO2_W2Drive = 0x303;
		m_CanOpenIDParam.TxSDO_W2Drive = 0x583;
		m_CanOpenIDParam.RxSDO_W2Drive = 0x603;
	}
	// Wheel 2
	// can adresse motor 3
	if(m_iNumMotors >= 3)
	{
		m_CanOpenIDParam.TxPDO1_W3Drive = 0x184;
		m_CanOpenIDParam.TxPDO2_W3Drive = 0x284;
		m_CanOpenIDParam.RxPDO2_W3Drive = 0x304;
		m_CanOpenIDParam.TxSDO_W3Drive = 0x584;
		m_CanOpenIDParam.RxSDO_W3Drive = 0x604;
	}
	// can adresse motor 4
	if(m_iNumMotors >= 4)
	{
		m_CanOpenIDParam.TxPDO1_W4Drive = 0x185;
		m_CanOpenIDParam.TxPDO2_W4Drive = 0x285;
		m_CanOpenIDParam.RxPDO2_W4Drive = 0x305;
		m_CanOpenIDParam.TxSDO_W4Drive = 0x585;
		m_CanOpenIDParam.RxSDO_W4Drive = 0x605;
	}
}

//-----------------------------------------------
CanCtrlPltfCOb3::~CanCtrlPltfCOb3()
{

	if (m_pCanCtrl != NULL)
	{
		delete m_pCanCtrl;
	}

	for(unsigned int i = 0; i < m_vpMotor.size(); i++)
	{
		if (m_vpMotor[i] != NULL)
		{
			delete m_vpMotor[i];
		}
	}

}

//-----------------------------------------------
void CanCtrlPltfCOb3::readConfiguration()
{

	int iTypeCan = 0;
	int iMaxMessages = 0;

	DriveParam DriveParamW1DriveMotor;
	DriveParam DriveParamW2DriveMotor;
	DriveParam DriveParamW3DriveMotor;
	DriveParam DriveParamW4DriveMotor;
	
	std::string strTypeDrive;

	double dScaleToMM;

	// read Platform.ini (Coupling of Drive/Steer for Homing)
	m_IniFile.SetFileName(sIniDirectory + "Platform.ini", "CanCtrlPltfCOb3.cpp");

	m_IniFile.GetKeyInt("Geom", "RadiusWheel", &m_Param.iRadiusWheelMM, true);
	m_IniFile.GetKeyInt("Geom", "DistSteerAxisToDriveWheelCenter", &m_Param.iDistSteerAxisToDriveWheelMM, true);

	// read CanCtrl.ini
	m_IniFile.SetFileName(sIniDirectory + "CanCtrl.ini", "CanCtrlPltfCOb3.cpp");

	std::cout << "Can configuration of the platform:" << std::endl;
	
	// read Configuration of the Can-Network (CanCtrl.ini)
	m_IniFile.GetKeyInt("TypeCan", "Can", &iTypeCan, true);
	if (iTypeCan == 0)
	{
		sComposed = sIniDirectory;
		sComposed += "CanCtrl.ini";
		m_pCanCtrl = new CanPeakSys(sComposed.c_str());
		std::cout << "Uses CAN-Peak-Systems dongle" << std::endl;
	}
	else if (iTypeCan == 1)
	{
		sComposed = sIniDirectory;
		sComposed += "CanCtrl.ini";
		m_pCanCtrl = new CANPeakSysUSB(sComposed.c_str());
		std::cout << "Uses CAN-Peak-USB" << std::endl;
	}
	else if (iTypeCan == 2)
	{
		sComposed = sIniDirectory;
		sComposed += "CanCtrl.ini";
		m_pCanCtrl = new CanESD(sComposed.c_str(), false);
		std::cout << "Uses CAN-ESD-card" << std::endl;
	}

	// CanOpenId's ----- Default values (DESIRE)
	// Wheel 1
	// DriveMotor
	if(m_iNumDrives >= 1)
	{
		m_IniFile.GetKeyInt("CanOpenIDs", "TxPDO1_W1Drive", &m_CanOpenIDParam.TxPDO1_W1Drive, true);
		m_IniFile.GetKeyInt("CanOpenIDs", "TxPDO2_W1Drive", &m_CanOpenIDParam.TxPDO2_W1Drive, true);
		m_IniFile.GetKeyInt("CanOpenIDs", "RxPDO2_W1Drive", &m_CanOpenIDParam.RxPDO2_W1Drive, true);
		m_IniFile.GetKeyInt("CanOpenIDs", "TxSDO_W1Drive", &m_CanOpenIDParam.TxSDO_W1Drive, true);
		m_IniFile.GetKeyInt("CanOpenIDs", "RxSDO_W1Drive", &m_CanOpenIDParam.RxSDO_W1Drive, true);
	}

	// Wheel 2
	// DriveMotor
	if(m_iNumDrives >= 2)
	{
		m_IniFile.GetKeyInt("CanOpenIDs", "TxPDO1_W2Drive", &m_CanOpenIDParam.TxPDO1_W2Drive, true);
		m_IniFile.GetKeyInt("CanOpenIDs", "TxPDO2_W2Drive", &m_CanOpenIDParam.TxPDO2_W2Drive, true);
		m_IniFile.GetKeyInt("CanOpenIDs", "RxPDO2_W2Drive", &m_CanOpenIDParam.RxPDO2_W2Drive, true);
		m_IniFile.GetKeyInt("CanOpenIDs", "TxSDO_W2Drive", &m_CanOpenIDParam.TxSDO_W2Drive, true);
		m_IniFile.GetKeyInt("CanOpenIDs", "RxSDO_W2Drive", &m_CanOpenIDParam.RxSDO_W2Drive, true);
	}
	
	// Wheel 3
	// DriveMotor
	if(m_iNumDrives >= 3)
	{
		m_IniFile.GetKeyInt("CanOpenIDs", "TxPDO1_W3Drive", &m_CanOpenIDParam.TxPDO1_W3Drive, true);
		m_IniFile.GetKeyInt("CanOpenIDs", "TxPDO2_W3Drive", &m_CanOpenIDParam.TxPDO2_W3Drive, true);
		m_IniFile.GetKeyInt("CanOpenIDs", "RxPDO2_W3Drive", &m_CanOpenIDParam.RxPDO2_W3Drive, true);
		m_IniFile.GetKeyInt("CanOpenIDs", "TxSDO_W3Drive", &m_CanOpenIDParam.TxSDO_W3Drive, true);
		m_IniFile.GetKeyInt("CanOpenIDs", "RxSDO_W3Drive", &m_CanOpenIDParam.RxSDO_W3Drive, true);
	}
	
	// Wheel 4
	// DriveMotor
	if(m_iNumDrives >= 4)
	{
		m_IniFile.GetKeyInt("CanOpenIDs", "TxPDO1_W4Drive", &m_CanOpenIDParam.TxPDO1_W4Drive, true);
		m_IniFile.GetKeyInt("CanOpenIDs", "TxPDO2_W4Drive", &m_CanOpenIDParam.TxPDO2_W4Drive, true);
		m_IniFile.GetKeyInt("CanOpenIDs", "RxPDO2_W4Drive", &m_CanOpenIDParam.RxPDO2_W4Drive, true);
		m_IniFile.GetKeyInt("CanOpenIDs", "TxSDO_W4Drive", &m_CanOpenIDParam.TxSDO_W4Drive, true);
		m_IniFile.GetKeyInt("CanOpenIDs", "RxSDO_W4Drive", &m_CanOpenIDParam.RxSDO_W4Drive, true);
	}


	// read configuration of the Drives (CanCtrl.ini)
	/* Drivemotor1-Parameters Old
	m_IniFile.GetKeyString("TypeDrive", "Drive1", &strTypeDrive, true);	
	m_IniFile.GetKeyInt(strTypeDrive.c_str(), "EncIncrPerRevMot", &(m_GearMotDrive1.iEncIncrPerRevMot), true);
	m_IniFile.GetKeyDouble(strTypeDrive.c_str(), "VelMeasFrqHz", &(m_GearMotDrive1.dVelMeasFrqHz), true);
	m_IniFile.GetKeyDouble(strTypeDrive.c_str(), "BeltRatio", &(m_GearMotDrive1.dBeltRatio), true);
	m_IniFile.GetKeyDouble(strTypeDrive.c_str(), "GearRatio", &(m_GearMotDrive1.dGearRatio), true);
	m_IniFile.GetKeyInt(strTypeDrive.c_str(), "Sign", &(m_GearMotDrive1.iSign), true);
	m_IniFile.GetKeyDouble(strTypeDrive.c_str(), "VelMaxEncIncrS", &(m_GearMotDrive1.dVelMaxEncIncrS), true);
	m_IniFile.GetKeyDouble(strTypeDrive.c_str(), "AccIncrS", &(m_GearMotDrive1.dAccIncrS2), true);
	m_IniFile.GetKeyDouble(strTypeDrive.c_str(), "DecIncrS", &(m_GearMotDrive1.dDecIncrS2), true);
	m_IniFile.GetKeyDouble(strTypeDrive.c_str(), "EncOffsetIncr",&(m_GearMotDrive1.iEncOffsetIncr),true);
	*/

	// "Drive Motor Type1" drive parameters	
	if(m_iNumDrives >= 1)
	{
		m_IniFile.GetKeyInt("Drive1", "EncIncrPerRevMot", &(m_GearMotDrive1.iEncIncrPerRevMot), true);
		m_IniFile.GetKeyDouble("Drive1", "VelMeasFrqHz", &(m_GearMotDrive1.dVelMeasFrqHz), true);
		m_IniFile.GetKeyDouble("Drive1", "BeltRatio", &(m_GearMotDrive1.dBeltRatio), true);
		m_IniFile.GetKeyDouble("Drive1", "GearRatio", &(m_GearMotDrive1.dGearRatio), true);
		m_IniFile.GetKeyInt("Drive1", "Sign", &(m_GearMotDrive1.iSign), true);
		m_IniFile.GetKeyDouble("Drive1", "VelMaxEncIncrS", &(m_GearMotDrive1.dVelMaxEncIncrS), true);
		m_IniFile.GetKeyDouble("Drive1", "AccIncrS", &(m_GearMotDrive1.dAccIncrS2), true);
		m_IniFile.GetKeyDouble("Drive1", "DecIncrS", &(m_GearMotDrive1.dDecIncrS2), true);
		m_IniFile.GetKeyInt("Drive1", "EncOffsetIncr",&(m_GearMotDrive1.iEncOffsetIncr),true);
		m_IniFile.GetKeyBool("Drive1", "IsSteering", &(m_GearMotDrive1.bIsSteer), true);
		m_IniFile.GetKeyDouble("Drive1", "CurrentToTorque", &(m_GearMotDrive1.dCurrentToTorque), false);
		m_IniFile.GetKeyDouble("Drive1", "CurrMax", &(m_GearMotDrive1.dCurrMax), false);
		m_IniFile.GetKeyInt("Drive1", "HomingDigIn", &(m_GearMotDrive1.iHomingDigIn), false);
	}

	// "Drive Motor Type2" drive parameters	
	if(m_iNumDrives >= 2)
	{
		m_IniFile.GetKeyInt("Drive2", "EncIncrPerRevMot", &(m_GearMotDrive2.iEncIncrPerRevMot), true);
		m_IniFile.GetKeyDouble("Drive2", "VelMeasFrqHz", &(m_GearMotDrive2.dVelMeasFrqHz), true);
		m_IniFile.GetKeyDouble("Drive2", "BeltRatio", &(m_GearMotDrive2.dBeltRatio), true);
		m_IniFile.GetKeyDouble("Drive2", "GearRatio", &(m_GearMotDrive2.dGearRatio), true);
		m_IniFile.GetKeyInt("Drive2", "Sign", &(m_GearMotDrive2.iSign), true);
		m_IniFile.GetKeyDouble("Drive2", "VelMaxEncIncrS", &(m_GearMotDrive2.dVelMaxEncIncrS), true);
		m_IniFile.GetKeyDouble("Drive2", "AccIncrS", &(m_GearMotDrive2.dAccIncrS2), true);
		m_IniFile.GetKeyDouble("Drive2", "DecIncrS", &(m_GearMotDrive2.dDecIncrS2), true);
		m_IniFile.GetKeyInt("Drive2", "EncOffsetIncr",&(m_GearMotDrive2.iEncOffsetIncr),true);
		m_IniFile.GetKeyBool("Drive2", "IsSteering", &(m_GearMotDrive2.bIsSteer), true);
		m_IniFile.GetKeyDouble("Drive2", "CurrentToTorque", &(m_GearMotDrive2.dCurrentToTorque), false);
		m_IniFile.GetKeyDouble("Drive2", "CurrMax", &(m_GearMotDrive2.dCurrMax), false);
		m_IniFile.GetKeyInt("Drive2", "HomingDigIn", &(m_GearMotDrive2.iHomingDigIn), false);
	}

	// "Drive Motor Type3" drive parameters	
	if(m_iNumDrives >= 3)
	{
		m_IniFile.GetKeyInt("Drive3", "EncIncrPerRevMot", &(m_GearMotDrive3.iEncIncrPerRevMot), true);
		m_IniFile.GetKeyDouble("Drive3", "VelMeasFrqHz", &(m_GearMotDrive3.dVelMeasFrqHz), true);
		m_IniFile.GetKeyDouble("Drive3", "BeltRatio", &(m_GearMotDrive3.dBeltRatio), true);
		m_IniFile.GetKeyDouble("Drive3", "GearRatio", &(m_GearMotDrive3.dGearRatio), true);
		m_IniFile.GetKeyInt("Drive3", "Sign", &(m_GearMotDrive3.iSign), true);
		m_IniFile.GetKeyDouble("Drive3", "VelMaxEncIncrS", &(m_GearMotDrive3.dVelMaxEncIncrS), true);
		m_IniFile.GetKeyDouble("Drive3", "AccIncrS", &(m_GearMotDrive3.dAccIncrS2), true);
		m_IniFile.GetKeyDouble("Drive3", "DecIncrS", &(m_GearMotDrive3.dDecIncrS2), true);
		m_IniFile.GetKeyInt("Drive3", "EncOffsetIncr",&(m_GearMotDrive3.iEncOffsetIncr),true);
		m_IniFile.GetKeyBool("Drive3", "IsSteering", &(m_GearMotDrive3.bIsSteer), true);
		m_IniFile.GetKeyDouble("Drive3", "CurrentToTorque", &(m_GearMotDrive3.dCurrentToTorque), false);
		m_IniFile.GetKeyDouble("Drive3", "CurrMax", &(m_GearMotDrive3.dCurrMax), false);
		m_IniFile.GetKeyInt("Drive3", "HomingDigIn", &(m_GearMotDrive3.iHomingDigIn), false);
	}

	// "Drive Motor Type4" drive parameters	
	if(m_iNumDrives == 4)
	{
		m_IniFile.GetKeyInt("Drive4", "EncIncrPerRevMot", &(m_GearMotDrive4.iEncIncrPerRevMot), true);
		m_IniFile.GetKeyDouble("Drive4", "VelMeasFrqHz", &(m_GearMotDrive4.dVelMeasFrqHz), true);
		m_IniFile.GetKeyDouble("Drive4", "BeltRatio", &(m_GearMotDrive4.dBeltRatio), true);
		m_IniFile.GetKeyDouble("Drive4", "GearRatio", &(m_GearMotDrive4.dGearRatio), true);
		m_IniFile.GetKeyInt("Drive4", "Sign", &(m_GearMotDrive4.iSign), true);
		m_IniFile.GetKeyDouble("Drive4", "VelMaxEncIncrS", &(m_GearMotDrive4.dVelMaxEncIncrS), true);
		m_IniFile.GetKeyDouble("Drive4", "AccIncrS", &(m_GearMotDrive4.dAccIncrS2), true);
		m_IniFile.GetKeyDouble("Drive4", "DecIncrS", &(m_GearMotDrive4.dDecIncrS2), true);
		m_IniFile.GetKeyInt("Drive4", "EncOffsetIncr",&(m_GearMotDrive4.iEncOffsetIncr),true);
		m_IniFile.GetKeyBool("Drive4", "IsSteering", &(m_GearMotDrive4.bIsSteer), true);
		m_IniFile.GetKeyDouble("Drive4", "CurrentToTorque", &(m_GearMotDrive4.dCurrentToTorque), false);
		m_IniFile.GetKeyDouble("Drive4", "CurrMax", &(m_GearMotDrive4.dCurrMax), false);
		m_IniFile.GetKeyInt("Drive4", "HomingDigIn", &(m_GearMotDrive4.iHomingDigIn), false);
	}

	if(m_iNumDrives >= 1)
	{
		DriveParamW1DriveMotor.setParam(
			0,
			m_GearMotDrive1.iEncIncrPerRevMot,
			m_GearMotDrive1.dVelMeasFrqHz,
			m_GearMotDrive1.dBeltRatio,
			m_GearMotDrive1.dGearRatio,
			m_GearMotDrive1.iSign,
			m_GearMotDrive1.dVelMaxEncIncrS,
			m_GearMotDrive1.dAccIncrS2,
			m_GearMotDrive1.dDecIncrS2,
			m_GearMotDrive1.iEncOffsetIncr,
			m_GearMotDrive1.bIsSteer,
			m_GearMotDrive1.dCurrentToTorque,
			m_GearMotDrive1.dCurrMax,
			m_GearMotDrive1.iHomingDigIn);
	}
	
	if(m_iNumDrives >= 2)
	{
		DriveParamW2DriveMotor.setParam(
			2,
			m_GearMotDrive2.iEncIncrPerRevMot,
			m_GearMotDrive2.dVelMeasFrqHz,
			m_GearMotDrive2.dBeltRatio,
			m_GearMotDrive2.dGearRatio,
			m_GearMotDrive2.iSign,
			m_GearMotDrive2.dVelMaxEncIncrS,
			m_GearMotDrive2.dAccIncrS2,
			m_GearMotDrive2.dDecIncrS2,
			m_GearMotDrive2.iEncOffsetIncr,
			m_GearMotDrive2.bIsSteer,
			m_GearMotDrive2.dCurrentToTorque,
			m_GearMotDrive2.dCurrMax,
			m_GearMotDrive2.iHomingDigIn);
	}

	if(m_iNumDrives >= 3)
	{
		DriveParamW3DriveMotor.setParam(
			4,
			m_GearMotDrive3.iEncIncrPerRevMot,
			m_GearMotDrive3.dVelMeasFrqHz,
			m_GearMotDrive3.dBeltRatio,
			m_GearMotDrive3.dGearRatio,
			m_GearMotDrive3.iSign,
			m_GearMotDrive3.dVelMaxEncIncrS,
			m_GearMotDrive3.dAccIncrS2,
			m_GearMotDrive3.dDecIncrS2,
			m_GearMotDrive3.iEncOffsetIncr,
			m_GearMotDrive3.bIsSteer,
			m_GearMotDrive3.dCurrentToTorque,
			m_GearMotDrive3.dCurrMax,
			m_GearMotDrive3.iHomingDigIn);
	}
	
	if(m_iNumDrives >= 4)
	{
		DriveParamW4DriveMotor.setParam(
			6,
			m_GearMotDrive4.iEncIncrPerRevMot,
			m_GearMotDrive4.dVelMeasFrqHz,
			m_GearMotDrive4.dBeltRatio,
			m_GearMotDrive4.dGearRatio,
			m_GearMotDrive4.iSign,
			m_GearMotDrive4.dVelMaxEncIncrS,
			m_GearMotDrive4.dAccIncrS2,
			m_GearMotDrive4.dDecIncrS2,
			m_GearMotDrive4.iEncOffsetIncr,
			m_GearMotDrive4.bIsSteer,
			m_GearMotDrive4.dCurrentToTorque,
			m_GearMotDrive4.dCurrMax,
			m_GearMotDrive4.iHomingDigIn);
	}


	// read Platform.ini
	m_IniFile.SetFileName(sIniDirectory + "Platform.ini", "CanCtrlPltfCOb3.cpp");


	// ------ WHEEL 1 ------ //
	// --- Motor Wheel 1 Drive
	if(m_iNumDrives >= 1)
	{
		m_IniFile.GetKeyInt("Config", "Wheel1DriveMotor", &m_Param.iHasWheel1DriveMotor, true);
		if (m_Param.iHasWheel1DriveMotor == 0)
		{
			// No motor
			std::cout << "node Wheel1DriveMotor available = 0" << std::endl;
		}
		else
		{
			// Motor Harmonica
			std::cout << "Wheel1DriveMotor available = type version 2" << std::endl;
			m_vpMotor[0] = new CanDriveHarmonica();
			((CanDriveHarmonica*) m_vpMotor[0])->setCanOpenParam(
				m_CanOpenIDParam.TxPDO1_W1Drive, m_CanOpenIDParam.TxPDO2_W1Drive, m_CanOpenIDParam.RxPDO2_W1Drive,
				m_CanOpenIDParam.TxSDO_W1Drive, m_CanOpenIDParam.RxSDO_W1Drive);
			m_vpMotor[0]->setCanItf(m_pCanCtrl);
			m_vpMotor[0]->setDriveParam(DriveParamW1DriveMotor);
		}
	}

	
	// ------ WHEEL 2 ------ //
	// --- Motor Wheel 2 Drive
	if(m_iNumDrives >= 2)
	{
		m_IniFile.GetKeyInt("Config", "Wheel2DriveMotor", &m_Param.iHasWheel2DriveMotor, true);
		if (m_Param.iHasWheel2DriveMotor == 0)
		{
			// No motor
			std::cout << "node Wheel2DriveMotor available = 0" << std::endl;
		}
		else
		{
			// Motor Harmonica
			std::cout << "Wheel2DriveMotor available = type version 2" << std::endl;
			m_vpMotor[1] = new CanDriveHarmonica();
			((CanDriveHarmonica*) m_vpMotor[1])->setCanOpenParam(
				m_CanOpenIDParam.TxPDO1_W2Drive, m_CanOpenIDParam.TxPDO2_W2Drive, m_CanOpenIDParam.RxPDO2_W2Drive,
				m_CanOpenIDParam.TxSDO_W2Drive, m_CanOpenIDParam.RxSDO_W2Drive);
			m_vpMotor[1]->setCanItf(m_pCanCtrl);
			m_vpMotor[1]->setDriveParam(DriveParamW2DriveMotor);
		}
	}

	
	// ------ WHEEL 3 ------ //
	// --- Motor Wheel 3 Drive
	if(m_iNumDrives >= 3)
	{
		m_IniFile.GetKeyInt("Config", "Wheel3DriveMotor", &m_Param.iHasWheel3DriveMotor, true);
		if (m_Param.iHasWheel3DriveMotor == 0)
		{
			// No motor
			std::cout << "node Wheel3DriveMotor available = 0" << std::endl;
		}
		else
		{
			// Motor Harmonica
			std::cout << "Wheel3DriveMotor available = type version 2" << std::endl;
			m_vpMotor[2] = new CanDriveHarmonica();
			((CanDriveHarmonica*) m_vpMotor[2])->setCanOpenParam(
				m_CanOpenIDParam.TxPDO1_W3Drive, m_CanOpenIDParam.TxPDO2_W3Drive, m_CanOpenIDParam.RxPDO2_W3Drive,
				m_CanOpenIDParam.TxSDO_W3Drive, m_CanOpenIDParam.RxSDO_W3Drive);
			m_vpMotor[2]->setCanItf(m_pCanCtrl);
			m_vpMotor[2]->setDriveParam(DriveParamW3DriveMotor);
		}
	}

	
	// ------ WHEEL 4 ------ //
	// --- Motor Wheel 4 Drive
	if(m_iNumDrives == 4)
	{
		m_IniFile.GetKeyInt("Config", "Wheel4DriveMotor", &m_Param.iHasWheel4DriveMotor, true);
		if (m_Param.iHasWheel4DriveMotor == 0)
		{
			// No motor
			std::cout << "node Wheel4DriveMotor available = 0" << std::endl;
		}
		else
		{
			// Motor Harmonica
			std::cout << "Wheel4DriveMotor available = type version 2" << std::endl;
			m_vpMotor[3] = new CanDriveHarmonica();
			((CanDriveHarmonica*) m_vpMotor[3])->setCanOpenParam(
				m_CanOpenIDParam.TxPDO1_W4Drive, m_CanOpenIDParam.TxPDO2_W4Drive, m_CanOpenIDParam.RxPDO2_W4Drive,
				m_CanOpenIDParam.TxSDO_W4Drive, m_CanOpenIDParam.RxSDO_W4Drive);
			m_vpMotor[3]->setCanItf(m_pCanCtrl);
			m_vpMotor[3]->setDriveParam(DriveParamW4DriveMotor);
		}
	}


	m_IniFile.GetKeyInt("Config", "GenericBufferLen", &iMaxMessages, true);


}

//-----------------------------------------------
int CanCtrlPltfCOb3::evalCanBuffer()
{
	bool bRet;
//	char cBuf[200];
	
	m_Mutex.lock();

	// as long as there is something in the can buffer -> read out next message
	while(m_pCanCtrl->receiveMsg(&m_CanMsgRec) == true)
	{
		bRet = false;
		// check for every motor if message belongs to it
		for (unsigned int i = 0; i < m_vpMotor.size(); i++)
		{
			// if message belongs to this motor write data (Pos, Vel, ...) to internal member vars
			bRet |= m_vpMotor[i]->evalReceivedMsg(m_CanMsgRec);
		}

		if (bRet == false)
		{
			std::cout << "evalCanBuffer(): Received CAN_Message with unknown identifier " << m_CanMsgRec.m_iID << std::endl;
		}		
	};
	

	m_Mutex.unlock();

	return 0;
}

//-----------------------------------------------
bool CanCtrlPltfCOb3::initPltf()
{	
	// read Configuration parameters from Inifile
	readConfiguration();
		
	// Vectors for drive objects and return values
	std::vector<bool> vbRetDriveMotor;
	std::vector<CanDriveItf*> vpDriveMotor;
	bool bHomingOk;

	vbRetDriveMotor.assign(m_iNumDrives,0);

	// Homing is done on a wheel-module base (steering and driving needs to be synchronized) 
	// copy Motor-Pointer into Steer/Drive vector for more insight
	for(int i=0; i<=m_iNumMotors; i++)
		vpDriveMotor.push_back(m_vpMotor[i]);

	std::vector<double> vdFactorVel;
//	vdFactorVel.assign(4,0);
	vdFactorVel.assign(m_iNumDrives,0);
	double dhomeVeloRadS = -1.0;


	// Start can open network
	std::cout << "StartCanOpen" << std::endl;
	sendNetStartCanOpen();
	

	// initialize drives
	
	// 1st init watchdogs
	std::cout << "Initialization of Watchdogs" << std::endl;
	for(int i=0; i<m_iNumMotors; i++)
	{

		m_vpMotor[i]->startWatchdog(true);
	}
	usleep(10000);
	
	// 2nd send watchdogs to bed while initializing drives
	for(int i=0; i<m_iNumMotors; i++)
	{
		m_vpMotor[i]->startWatchdog(false);
	}

	usleep(100000);

	std::cout << "Initialization of Watchdogs done" << std::endl;


	// ---------------------- start homing procedurs
	// Perform homing of all wheels simultaneously
	// o.k. to avoid crashing hardware -> lets check that we have at least the 8 motors, like we have on cob
	if( m_vpMotor.size() == m_iNumMotors )
	{
		// Initialize and start all motors
		for (int i = 0; i<m_iNumDrives; i++)
		{
std::cout<<i<<": init numDrives: "<<m_iNumDrives<<std::endl;
			// init + start
			vbRetDriveMotor[i] = vpDriveMotor[i]->init();
if(vbRetDriveMotor[i]) std::cout<<"motor initialized"<<std::endl;
			usleep(10000);
			vbRetDriveMotor[i] = vbRetDriveMotor[i] && vpDriveMotor[i]->start();
			usleep(10000);
			// output State / Errors
			if (vbRetDriveMotor[i])
				std::cout << "Initialization of Wheel "<< (i+1) << " OK" << std::endl;
			// Just to be sure: Set vel to zero
			vpDriveMotor[i]->setGearVelRadS(0);
		}	
	}
	for(int i=0; i<m_iNumMotors; i++)
	{
		m_vpMotor[i]->startWatchdog(true);
	}
	return (true);
}

//-----------------------------------------------
bool CanCtrlPltfCOb3::resetPltf()
{
	bool bRetMotor = true;
	bool bRet = true;
	
	for(unsigned int i = 0; i < m_vpMotor.size(); i++)
	{
		bRetMotor = m_vpMotor[i]->start();
		if (bRetMotor == true)
		{
			m_vpMotor[i]->setGearVelRadS(0);
		}
		else
		{
			std::cout << "Resetting of Motor " << i << " failed" << std::endl;
		}

		bRet &= bRetMotor;
	}
	return(bRet);
}

//-----------------------------------------------
bool CanCtrlPltfCOb3::shutdownPltf()
{

	for(unsigned int i = 0; i < m_vpMotor.size(); i++)
	{
		m_vpMotor[i]->shutdown();
	}

	return true;
}

//-----------------------------------------------
bool CanCtrlPltfCOb3::isPltfError()
{

	bool bErrMotor = false;
	std::vector<bool> vbErrMotor;
	vbErrMotor.resize(m_vpMotor.size());

	// check all motors for errors
	for(unsigned int i = 0; i < m_vpMotor.size(); i++)
	{
		vbErrMotor[i] = m_vpMotor[i]->isError();
		// if watchdog is not yet in error state but one motor is errorneous Log Out Error-Msg
		if( (m_bWatchdogErr == false) && (vbErrMotor[i] == true) )
		{
			std::cout << "Motor " << i << " error" << std::endl;
		}
		// check whether no motor was errorneous
		bErrMotor |= vbErrMotor[i];
	}

	// if no motor is in error state force Watchdog to "No Error" state
	if(bErrMotor == false)
		m_bWatchdogErr = false;
	else
		m_bWatchdogErr = true;

	if (m_bWatchdogErr) return true;


	// Check communication
	double dWatchTime = 0;

	for(unsigned int i = 0; i < m_vpMotor.size(); i++)
	{
		dWatchTime = m_vpMotor[i]->getTimeToLastMsg();

		if(dWatchTime <  m_Param.dCanTimeout)
		{
			m_bWatchdogErr = false;
		}
		if( (dWatchTime > m_Param.dCanTimeout) && (m_bWatchdogErr == false) )
		{
			std::cout << "timeout CAN motor " << i << std::endl;
			m_bWatchdogErr = true;
			return true;
		}
	}

	return false;
}

//-----------------------------------------------
bool CanCtrlPltfCOb3::startWatchdog(bool bStarted)
{

	bool bRet = true;

	for(unsigned int i = 0; i < m_vpMotor.size(); i++)
	{
		bRet = m_vpMotor[i]->startWatchdog(bStarted);
	}

	return (bRet);
}

//-----------------------------------------------
void CanCtrlPltfCOb3::sendNetStartCanOpen()
{
	CanMsg msg;

	msg.m_iID  = 0;
	msg.m_iLen = 2;
	msg.set(1,0,0,0,0,0,0,0);
	m_pCanCtrl->transmitMsg(msg, false);

	usleep(100000);
}


//-----------------------------------------------
// Motor Controlers
//-----------------------------------------------


//-----------------------------------------------
int CanCtrlPltfCOb3::setVelGearRadS(int iCanIdent, double dVelGearRadS)
{		
	m_Mutex.lock();

	// If an error was detected and processed in isPltfErr().
	if (m_bWatchdogErr == true)
	{
		// Error -> Stop motor driving
		std::cerr<<"watchdog error"<<std::endl;
		dVelGearRadS = 0;
		// The error is detected and quitted in the funtion isPltfErr().
	}

	for(unsigned int i = 0; i < m_vpMotor.size(); i++)
	{
		// check if Identifier fits to availlable hardware
		if(iCanIdent == m_viMotorID[i])
		{
			m_vpMotor[i]->setGearVelRadS(dVelGearRadS);
		}
	}
	
	m_Mutex.unlock();
	
	return 0;
}

//-----------------------------------------------
int CanCtrlPltfCOb3::requestMotPosVel(int iCanIdent)
{
	m_Mutex.lock();

	for(unsigned int i = 0; i < m_vpMotor.size(); i++)
	{
		// check if Identifier fits to availlable hardware
		if(iCanIdent == m_viMotorID[i])
		{
			m_vpMotor[i]->requestPosVel();
		}
	}
	
	m_Mutex.unlock();

	return 0;
}

//-----------------------------------------------
void CanCtrlPltfCOb3::requestDriveStatus()
{
	m_Mutex.lock();

	for(unsigned int i = 0; i < m_vpMotor.size(); i++)
	{
		m_vpMotor[i]->requestStatus();
	}

	m_Mutex.unlock();
}

//-----------------------------------------------
int CanCtrlPltfCOb3::getGearPosVelRadS(int iCanIdent, double* pdAngleGearRad, double* pdVelGearRadS)
{

	// init default outputs
	*pdAngleGearRad = 0;
	*pdVelGearRadS = 0;

	for(unsigned int i = 0; i < m_vpMotor.size(); i++)
	{
		// check if Identifier fits to availlable hardware
		if(iCanIdent == m_viMotorID[i])
		{
			m_vpMotor[i]->getGearPosVelRadS(pdAngleGearRad, pdVelGearRadS);
		}
	}
	
	return 0;
}

//-----------------------------------------------
int CanCtrlPltfCOb3::getGearDeltaPosVelRadS(int iCanIdent, double* pdAngleGearRad,
										   double* pdVelGearRadS)
{

	// init default outputs
	*pdAngleGearRad = 0;
	*pdVelGearRadS = 0;

	for(unsigned int i = 0; i < m_vpMotor.size(); i++)
	{
		// check if Identifier fits to availlable hardware
		if(iCanIdent == m_viMotorID[i])
		{
			m_vpMotor[i]->getGearDeltaPosVelRadS(pdAngleGearRad, pdVelGearRadS);
		}
	}

	return 0;
}

//-----------------------------------------------
void CanCtrlPltfCOb3::getStatus(int iCanIdent, int* piStatus, int* piTempCel)
{
	// init default outputs
	*piStatus = 0;
	*piTempCel = 0;

	for(unsigned int i = 0; i < m_vpMotor.size(); i++)
	{
		// check if Identifier fits to availlable hardware
		if(iCanIdent == m_viMotorID[i])
		{
			m_vpMotor[i]->getStatus(piStatus, piTempCel);
		}
	}

}
//-----------------------------------------------
void CanCtrlPltfCOb3::requestMotorTorque()
{
	m_Mutex.lock();

	for(unsigned int i = 0; i < m_vpMotor.size(); i++)
	{
		m_vpMotor[i]->requestMotorTorque();
	}

	m_Mutex.unlock();
}	

//-----------------------------------------------
void CanCtrlPltfCOb3::getMotorTorque(int iCanIdent, double* pdTorqueNm)
{
	// init default outputs
	*pdTorqueNm = 0;

	for(unsigned int i = 0; i < m_vpMotor.size(); i++)
	{
		// check if Identifier fits to availlable hardware
		if(iCanIdent == m_viMotorID[i])
		{
			m_vpMotor[i]->getMotorTorque(pdTorqueNm);
		}
	}

}
//-----------------------------------------------
void CanCtrlPltfCOb3::setMotorTorque(int iCanIdent, double dTorqueNm)
{
	m_Mutex.lock();

	for(unsigned int i = 0; i < m_vpMotor.size(); i++)
	{
		// check if Identifier fits to availlable hardware
		if(iCanIdent == m_viMotorID[i])
		{
			m_vpMotor[i]->setMotorTorque(dTorqueNm);
		}
	}

	m_Mutex.unlock();
}

//-----------------------------------------------
// Read out Elmo-Recorder via CAN (cpc-pk)
//-----------------------------------------------

//-----------------------------------------------
int CanCtrlPltfCOb3::ElmoRecordings(int iFlag, int iParam, std::string sString) {
	int tempRet = 0;
	int bRet = 0;
	
	switch(iFlag) {
		case 0: //Flag = 0 means reset recorder and configure it
			for(unsigned int i = 0; i < m_vpMotor.size(); i++) {
				m_vpMotor[i]->setRecorder(0, iParam); //Configure Elmo Recorder with RecordingGap and start immediately
			}
			return 0;

		case 1: //Flag = 1 means start readout process, mustn't be called too early (while Rec is in process..)
			for(unsigned int i = 0; i < m_vpMotor.size(); i++) {
				if(tempRet = m_vpMotor[i]->setRecorder(1, iParam, sString) > bRet) bRet = tempRet; //Query Readout of Index to Log Directory
			}
			return bRet;
		
		case 99:
			for(unsigned int i = 0; i < m_vpMotor.size(); i++) {
				m_vpMotor[i]->setRecorder(99, 0); //Stop any ongoing SDO transfer and clear corresponding data.
			}
			return 0;
			
		case 100:
			for(unsigned int i = 0; i < m_vpMotor.size(); i++) {
				bRet += m_vpMotor[i]->setRecorder(2, 0); //Request state of transmission
			}
			return bRet;
		
		default:
			return -1;
	}
}
